﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using FluxJpeg.Core;
using System.Windows.Media.Imaging;

namespace SilverlightCapture
{
    public partial class MainPage : UserControl
    {
        private SaveFileDialog saveDialog;
        CaptureSource _captureSource;
        WriteableBitmap _bmp = null;

        public MainPage()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(MainPage_Loaded);

            // セキュリティの制限(?)により、SaveFileDialogのデフォルトファイル名は
            // 設定できない
            this.saveDialog = new SaveFileDialog();
            this.saveDialog.DefaultExt = ".jpg";
            this.saveDialog.Filter = "JPEGファイル|*.jpg";
            this.saveDialog.FilterIndex = 1; 
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            // 接続されているカメラ一覧をバインド
            VideoSources.ItemsSource = CaptureDeviceConfiguration.GetAvailableVideoCaptureDevices();

            // カメラ選択ボタンを有効化
            buttonCameraSelect.IsEnabled = true;

            _captureSource = new CaptureSource();
            _captureSource.CaptureImageCompleted += ((s, args) =>
            {
                // _captureSource.CaptureImageAsync() が実行された後
                // 非同期に以下のコードが実行される
                _bmp = args.Result;

                // 保存ボタンを有効化
                buttonSave.IsEnabled = true;
            });
        }

        private void buttonCameraSelect_Click(object sender, RoutedEventArgs e)
        {
            if (_captureSource != null)
            {
                _captureSource.Stop(); // stop whatever device may be capturing

                // 選択された VideoCaptureDevice を割り当て
                _captureSource.VideoCaptureDevice = (VideoCaptureDevice)VideoSources.SelectedItem;

                // 利用可能な画像フォーマット(幅,高さ,fps)一覧をバインド
                var formats = _captureSource.VideoCaptureDevice.SupportedFormats;
                VideoFormats.ItemsSource = formats;
                
                // ボタンの有効/無効を設定
                buttonFormat.IsEnabled = true;
                buttonCapture.IsEnabled = false;
                buttonSave.IsEnabled = false;
            }
        }

        private void buttonFormat_Click(object sender, RoutedEventArgs e)
        {
            if (_captureSource != null)
            {
                _captureSource.VideoCaptureDevice.DesiredFormat = (VideoFormat)VideoFormats.SelectedItem;
                VideoBrush videoBrush = new VideoBrush();
                videoBrush.SetSource(_captureSource);
                WebcamCapture.Fill = videoBrush; // キャプチャ画像をWebcamCaptureへ描画

                // ユーザーにデバイスの利用許可を求める
                if (CaptureDeviceConfiguration.AllowedDeviceAccess || CaptureDeviceConfiguration.RequestDeviceAccess())
                {
                    // Webカメラの画像を描画開始
                    _captureSource.Start();

                    // キャプチャボタンを有効化
                    buttonCapture.IsEnabled = true;
                    buttonFormat.IsEnabled = false;
                }
            }
        }
        private void buttonCapture_Click(object sender, RoutedEventArgs e)
        {
            if (_captureSource != null)
            {
                // 画像を非同期にキャプチャ
                _captureSource.CaptureImageAsync();
            }

        }

        private void buttonSave_Click(object sender, RoutedEventArgs e)
        {
            bool? dialogResult = this.saveDialog.ShowDialog();

            if (dialogResult == true)
            {
                // WriteableBitmap を JPEGに変換して保存
                byte[] fileBytes = CreateImageAsJpeg(_bmp, 80);

                // ユーザーのボタン操作によってダイアログを出さないと
                // セキュリティ例外
                using (Stream fs = (Stream)this.saveDialog.OpenFile())
                {
                    fs.Write(fileBytes, 0, fileBytes.Length);
                    fs.Close();
                }
            }
        }

        /// <summary>
        /// ビットマップをJpegのバイト列へ変換します
        /// </summary>
        /// <param name="bitmap">変換元画像</param>
        /// <param name="quality">圧縮品質(0:最高圧縮率～100:最高品質)</param>
        /// <returns>Jpegエンコードされたバイト列</returns>
        private byte[] CreateImageAsJpeg(WriteableBitmap bitmap, int quality)
        {
            // このコードは
            // http://blogs.msdn.com/b/davrous/archive/2009/12/18/silverlight-4-tutorial-adding-avatar-support-to-the-wcf-ria-services-business-template.aspx
            // をほぼ転載しています

            int width = bitmap.PixelWidth;
            int height = bitmap.PixelHeight;
            int bands = 3;
            byte[][,] raster = new byte[bands][,];

            //Convert the Image to pass into FJCore
            //Code From http://stackoverflow.com/questions/1139200/using-fjcore-to-encode-silverlight-writeablebitmap
            for (int i = 0; i < bands; i++)
            {
                raster[i] = new byte[width, height];
            }

            for (int row = 0; row < height; row++)
            {
                for (int column = 0; column < width; column++)
                {
                    int pixel = bitmap.Pixels[width * row + column];
                    raster[0][column, row] = (byte)(pixel >> 16);
                    raster[1][column, row] = (byte)(pixel >> 8);
                    raster[2][column, row] = (byte)pixel;
                }
            }

            ColorModel model = new ColorModel { colorspace = ColorSpace.RGB };
            FluxJpeg.Core.Image img = new FluxJpeg.Core.Image(model, raster);

            //Encode the Image as a JPEG
            MemoryStream stream = new MemoryStream();
            FluxJpeg.Core.Encoder.JpegEncoder encoder = new FluxJpeg.Core.Encoder.JpegEncoder(img, quality, stream);
            encoder.Encode();

            //Back to the start
            stream.Seek(0, SeekOrigin.Begin);

            //Get the Bytes and write them to the stream
            byte[] binaryData = new Byte[stream.Length];
            long bytesRead = stream.Read(binaryData, 0, (int)stream.Length);
            return binaryData;
        }
    }
}
